@@ -29,7 +29,7 @@
 #include <champlain-gtk/champlain-gtk.h>
 #include <geoclue/geoclue-address.h>
 #include <geoclue/geoclue-position.h>
-#include <geocode-glib.h>
+#include <geocode-glib/geocode-glib.h>
 
 #include <clutter/clutter.h>
 
@@ -39,8 +39,6 @@
 
 #include "e-util/e-util.h"
 
-#include "e-contact-marker.h"
-
 #define E_CONTACT_MAP_GET_PRIVATE(obj) \
 	(G_TYPE_INSTANCE_GET_PRIVATE \
 	((obj), E_TYPE_CONTACT_MAP, EContactMapPrivate))
@@ -55,7 +53,7 @@ struct _EContactMapPrivate {
 
 struct _AsyncContext {
 	EContactMap *map;
-	EContactMarker *marker;
+	ClutterActor *marker;
 };
 
 enum {
@@ -73,31 +71,90 @@ G_DEFINE_TYPE (EContactMap, e_contact_map, GTK_CHAMPLAIN_TYPE_EMBED)
 static void
 async_context_free (AsyncContext *async_context)
 {
-	if (async_context->map != NULL)
-		g_object_unref (async_context->map);
+	g_clear_object (&async_context->map);
 
 	g_slice_free (AsyncContext, async_context);
 }
 
+static ClutterActor *
+texture_new_from_pixbuf (GdkPixbuf *pixbuf,
+                         GError **error)
+{
+	ClutterActor *texture = NULL;
+	const guchar *data;
+	gboolean has_alpha, success;
+	gint width, height, rowstride;
+	ClutterTextureFlags flags = 0;
+
+	data = gdk_pixbuf_get_pixels (pixbuf);
+	width = gdk_pixbuf_get_width (pixbuf);
+	height = gdk_pixbuf_get_height (pixbuf);
+	has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
+	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+
+	texture = clutter_texture_new ();
+	success = clutter_texture_set_from_rgb_data (
+		CLUTTER_TEXTURE (texture),
+		data, has_alpha, width, height, rowstride,
+		(has_alpha ? 4: 3), flags, NULL);
+
+	if (!success) {
+		clutter_actor_destroy (CLUTTER_ACTOR (texture));
+		texture = NULL;
+	}
+
+	return texture;
+}
+
+static ClutterActor *
+contact_map_photo_to_texture (EContactPhoto *photo)
+{
+	ClutterActor *texture = NULL;
+	GdkPixbuf *pixbuf = NULL;
+
+	if  (photo->type == E_CONTACT_PHOTO_TYPE_INLINED) {
+		GdkPixbufLoader *loader = gdk_pixbuf_loader_new ();
+
+		gdk_pixbuf_loader_write (
+			loader, photo->data.inlined.data,
+			photo->data.inlined.length, NULL);
+		gdk_pixbuf_loader_close (loader, NULL);
+		pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+		if (pixbuf != NULL)
+			g_object_ref (pixbuf);
+		g_object_unref (loader);
+
+	} else if (photo->type == E_CONTACT_PHOTO_TYPE_URI) {
+		pixbuf = gdk_pixbuf_new_from_file (photo->data.uri, NULL);
+	}
+
+	if (pixbuf != NULL) {
+		texture = texture_new_from_pixbuf (pixbuf, NULL);
+		g_object_unref (pixbuf);
+	}
+
+	return texture;
+}
+
 static void
-contact_map_address_resolved_cb (GObject *source,
+contact_map_address_resolved_cb (GObject *source_object,
                                  GAsyncResult *result,
                                  gpointer user_data)
 {
 	GHashTable *resolved = NULL;
 	gpointer marker_ptr;
-	const gchar *name;
-	gdouble latitude, longitude;
 	AsyncContext *async_context = user_data;
 	ChamplainMarkerLayer *marker_layer;
 	ChamplainMarker *marker;
-
-	g_return_if_fail (async_context != NULL);
-	g_return_if_fail (E_IS_CONTACT_MAP (async_context->map));
-	g_return_if_fail (E_IS_CONTACT_MARKER (async_context->marker));
+	GeocodePlace *place;
+	GeocodeLocation *location;
+	GList *search_results;
+	const gchar *name;
+	GError *local_error = NULL;
 
 	marker = CHAMPLAIN_MARKER (async_context->marker);
 	marker_layer = async_context->map->priv->marker_layer;
+	name = champlain_label_get_text (CHAMPLAIN_LABEL (marker));
 
 	/* If the marker_layer does not exist anymore, the map has
 	 * probably been destroyed before this callback was launched.
@@ -107,28 +164,46 @@ contact_map_address_resolved_cb (GObject *source,
 	if (!CHAMPLAIN_IS_MARKER_LAYER (marker_layer))
 		goto exit;
 
-	resolved = geocode_object_resolve_finish (
-		GEOCODE_OBJECT (source), result, NULL);
+	search_results = geocode_forward_search_finish (
+		GEOCODE_FORWARD (source_object), result, &local_error);
 
-	if (resolved == NULL ||
-	    !geocode_object_get_coords (resolved, &longitude, &latitude)) {
-		const gchar *name;
+	/* Sanity check. */
+	g_warn_if_fail (
+		((search_results != NULL) && (local_error == NULL)) ||
+		((search_results == NULL) && (local_error != NULL)));
 
-		name = champlain_label_get_text (CHAMPLAIN_LABEL (marker));
+	/* Keep quiet if the search just came up empty. */
+	if (g_error_matches (local_error, GEOCODE_ERROR, GEOCODE_ERROR_NO_MATCHES)) {
+		g_clear_error (&local_error);
+
+	/* Leave a breadcrumb on the console for any other errors. */
+	} else if (local_error != NULL) {
+		g_warning ("%s: %s", G_STRFUNC, local_error->message);
+		g_clear_error (&local_error);
+	}
+
+	if (search_results == NULL) {
 		g_signal_emit (
 			async_context->map,
 			signals[GEOCODING_FAILED], 0, name);
 		goto exit;
 	}
 
-	/* Move the marker to resolved position */
+	place = GEOCODE_PLACE (search_results->data);
+	location = geocode_place_get_location (place);
+
+	/* Move the marker to resolved position. */
 	champlain_location_set_location (
-		CHAMPLAIN_LOCATION (marker), latitude, longitude);
+		CHAMPLAIN_LOCATION (marker),
+		geocode_location_get_latitude (location),
+		geocode_location_get_longitude (location));
 	champlain_marker_layer_add_marker (marker_layer, marker);
 	champlain_marker_set_selected (marker, FALSE);
 
-	/* Store the marker in the hash table. Use it's label as key */
-	name = champlain_label_get_text (CHAMPLAIN_LABEL (marker));
+	/* Do not unref the list elements. */
+	g_list_free (search_results);
+
+	/* Store the marker in the hash table, using its label as key. */
 	marker_ptr = g_hash_table_lookup (
 		async_context->map->priv->markers, name);
 	if (marker_ptr != NULL) {
@@ -151,46 +226,38 @@ exit:
 }
 
 static void
-resolve_marker_position (EContactMap *map,
-                         EContactMarker *marker,
-                         EContactAddress *address)
+add_attr (GHashTable *hash_table,
+          const gchar *key,
+          const gchar *string)
 {
-	GeocodeObject *geocoder;
-	AsyncContext *async_context;
-	const gchar *key;
+	GValue *value;
 
-	g_return_if_fail (E_IS_CONTACT_MAP (map));
-	g_return_if_fail (address != NULL);
+	value = g_new0 (GValue, 1);
+	g_value_init (value, G_TYPE_STRING);
+	g_value_set_static_string (value, string);
 
-	geocoder = geocode_object_new ();
+	g_hash_table_insert (hash_table, g_strdup (key), value);
+}
 
-	key = GEOCODE_OBJECT_FIELD_POSTAL;
-	geocode_object_add (geocoder, key, address->code);
+static GHashTable *
+address_to_xep (EContactAddress *address)
+{
 
-	key = GEOCODE_OBJECT_FIELD_COUNTRY;
-	geocode_object_add (geocoder, key, address->country);
+	GHashTable *hash_table;
 
-	key = GEOCODE_OBJECT_FIELD_STATE;
-	geocode_object_add (geocoder, key, address->region);
+	hash_table = g_hash_table_new_full (
+		(GHashFunc) g_str_hash,
+		(GEqualFunc) g_str_equal,
+		(GDestroyNotify) g_free,
+		(GDestroyNotify) g_free);
 
-	key = GEOCODE_OBJECT_FIELD_CITY;
-	geocode_object_add (geocoder, key, address->locality);
+	add_attr (hash_table, "postalcode", address->code);
+	add_attr (hash_table, "country", address->country);
+	add_attr (hash_table, "region", address->region);
+	add_attr (hash_table, "locality", address->locality);
+	add_attr (hash_table, "street", address->street);
 
-	key = GEOCODE_OBJECT_FIELD_STREET;
-	geocode_object_add (geocoder, key, address->street);
-
-	async_context = g_slice_new0 (AsyncContext);
-	async_context->map = g_object_ref (map);
-	async_context->marker = marker;
-
-	geocode_object_resolve_async (
-		geocoder, NULL,
-		contact_map_address_resolved_cb,
-		async_context);
-
-	g_object_unref (geocoder);
-
-	g_signal_emit (map, signals[GEOCODING_STARTED], 0, marker);
+	return hash_table;
 }
 
 static void
@@ -198,12 +265,9 @@ contact_map_finalize (GObject *object)
 {
 	EContactMapPrivate *priv;
 
-	priv = E_CONTACT_MAP (object)->priv;
+	priv = E_CONTACT_MAP_GET_PRIVATE (object);
 
-	if (priv->markers) {
-		g_hash_table_destroy (priv->markers);
-		priv->markers = NULL;
-	}
+	g_hash_table_destroy (priv->markers);
 
 	/* Chain up to parent's finalize() method. */
 	G_OBJECT_CLASS (e_contact_map_parent_class)->finalize (object);
@@ -259,17 +323,16 @@ e_contact_map_class_init (EContactMapClass *class)
 static void
 e_contact_map_init (EContactMap *map)
 {
-	GHashTable *hash_table;
 	ChamplainMarkerLayer *layer;
 	ChamplainView *view;
 
 	map->priv = E_CONTACT_MAP_GET_PRIVATE (map);
 
-	hash_table = g_hash_table_new_full (
-		g_str_hash, g_str_equal,
-			(GDestroyNotify) g_free, NULL);
-
-	map->priv->markers = hash_table;
+	map->priv->markers = g_hash_table_new_full (
+		(GHashFunc) g_str_hash,
+		(GEqualFunc) g_str_equal,
+		(GDestroyNotify) g_free,
+		(GDestroyNotify) NULL);
 
 	view = gtk_champlain_embed_get_view (GTK_CHAMPLAIN_EMBED (map));
 	/* This feature is somehow broken sometimes, so disable it for now */
@@ -282,8 +345,7 @@ e_contact_map_init (EContactMap *map)
 GtkWidget *
 e_contact_map_new (void)
 {
-	return g_object_new (
-		E_TYPE_CONTACT_MAP,NULL);
+	return g_object_new (E_TYPE_CONTACT_MAP, NULL);
 }
 
 void
@@ -292,32 +354,43 @@ e_contact_map_add_contact (EContactMap *map,
 {
 	EContactAddress *address;
 	EContactPhoto *photo;
+	const gchar *contact_name;
 	const gchar *contact_uid;
-	gchar *name;
 
-	g_return_if_fail (map && E_IS_CONTACT_MAP (map));
-	g_return_if_fail (contact && E_IS_CONTACT (contact));
+	g_return_if_fail (E_IS_CONTACT_MAP (map));
+	g_return_if_fail (E_IS_CONTACT (contact));
 
 	photo = e_contact_get (contact, E_CONTACT_PHOTO);
+	contact_name = e_contact_get_const (contact, E_CONTACT_FILE_AS);
 	contact_uid = e_contact_get_const (contact, E_CONTACT_UID);
 
 	address = e_contact_get (contact, E_CONTACT_ADDRESS_HOME);
-	if (address) {
-		name = g_strconcat (e_contact_get_const (contact, E_CONTACT_FILE_AS), " (", _("Home"), ")", NULL);
-		e_contact_map_add_marker (map, name, contact_uid, address, photo);
+	if (address != NULL) {
+		gchar *name;
+
+		name = g_strdup_printf (
+			"%s (%s)", contact_name, _("Home"));
+		e_contact_map_add_marker (
+			map, name, contact_uid, address, photo);
 		g_free (name);
+
 		e_contact_address_free (address);
 	}
 
 	address = e_contact_get (contact, E_CONTACT_ADDRESS_WORK);
-	if (address) {
-		name = g_strconcat (e_contact_get_const (contact, E_CONTACT_FILE_AS), " (", _("Work"), ")", NULL);
-		e_contact_map_add_marker (map, name, contact_uid, address, photo);
+	if (address != NULL) {
+		gchar *name;
+
+		name = g_strdup_printf (
+			"%s (%s)", contact_name, _("Work"));
+		e_contact_map_add_marker (
+			map, name, contact_uid, address, photo);
 		g_free (name);
+
 		e_contact_address_free (address);
 	}
 
-	if (photo)
+	if (photo != NULL)
 		e_contact_photo_free (photo);
 }
 
@@ -328,16 +401,47 @@ e_contact_map_add_marker (EContactMap *map,
                           EContactAddress *address,
                           EContactPhoto *photo)
 {
-	EContactMarker *marker;
+	ClutterActor *marker;
+	GHashTable *hash_table;
+	GeocodeForward *geocoder;
+	AsyncContext *async_context;
 
-	g_return_if_fail (map && E_IS_CONTACT_MAP (map));
-	g_return_if_fail (name && *name);
-	g_return_if_fail (contact_uid && *contact_uid);
-	g_return_if_fail (address);
+	g_return_if_fail (E_IS_CONTACT_MAP (map));
+	g_return_if_fail (name != NULL);
+	g_return_if_fail (contact_uid != NULL);
+	g_return_if_fail (address != NULL);
 
-	marker = E_CONTACT_MARKER (e_contact_marker_new (name, contact_uid, photo));
+	marker = champlain_label_new ();
+	champlain_label_set_text (CHAMPLAIN_LABEL (marker), name);
 
-	resolve_marker_position (map, marker, address);
+	if (photo != NULL) {
+		champlain_label_set_image (
+			CHAMPLAIN_LABEL (marker),
+			contact_map_photo_to_texture (photo));
+	}
+
+	/* Stash the contact UID for EContactMapWindow. */
+	g_object_set_data_full (
+		G_OBJECT (marker), "contact-uid",
+		g_strdup (contact_uid),
+		(GDestroyNotify) g_free);
+
+	hash_table = address_to_xep (address);
+	geocoder = geocode_forward_new_for_params (hash_table);
+	g_hash_table_destroy (hash_table);
+
+	async_context = g_slice_new0 (AsyncContext);
+	async_context->map = g_object_ref (map);
+	async_context->marker = marker;
+
+	geocode_forward_search_async (
+		geocoder, NULL,
+		contact_map_address_resolved_cb,
+		async_context);
+
+	g_object_unref (geocoder);
+
+	g_signal_emit (map, signals[GEOCODING_STARTED], 0, marker);
 }
 
 /**
@@ -350,8 +454,8 @@ e_contact_map_remove_contact (EContactMap *map,
 {
 	ChamplainMarker *marker;
 
-	g_return_if_fail (map && E_IS_CONTACT_MAP (map));
-	g_return_if_fail (name && *name);
+	g_return_if_fail (E_IS_CONTACT_MAP (map));
+	g_return_if_fail (name != NULL);
 
 	marker = g_hash_table_lookup (map->priv->markers, name);
 
@@ -362,20 +466,6 @@ e_contact_map_remove_contact (EContactMap *map,
 	g_signal_emit (map, signals[CONTACT_REMOVED], 0, name);
 }
 
-void
-e_contact_map_remove_marker (EContactMap *map,
-                             ClutterActor *marker)
-{
-	const gchar *name;
-
-	g_return_if_fail (map && E_IS_CONTACT_MAP (map));
-	g_return_if_fail (marker && CLUTTER_IS_ACTOR (marker));
-
-	name = champlain_label_get_text (CHAMPLAIN_LABEL (marker));
-
-	e_contact_map_remove_contact (map, name);
-}
-
 void
 e_contact_map_zoom_on_marker (EContactMap *map,
                               ClutterActor *marker)
@@ -383,8 +473,8 @@ e_contact_map_zoom_on_marker (EContactMap *map,
 	ChamplainView *view;
 	gdouble lat, lng;
 
-	g_return_if_fail (map && E_IS_CONTACT_MAP (map));
-	g_return_if_fail (marker && CLUTTER_IS_ACTOR (marker));
+	g_return_if_fail (E_IS_CONTACT_MAP (map));
+	g_return_if_fail (CLUTTER_IS_ACTOR (marker));
 
 	lat = champlain_location_get_latitude (CHAMPLAIN_LOCATION (marker));
 	lng = champlain_location_get_longitude (CHAMPLAIN_LOCATION (marker));
